VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "IteratorDetected"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'      
Option Explicit

Private Const NO_CURRENT_ID = -1

Private selection_ As Word.Range
Private detector_ As Object

Private start_ As Long
Private data_ As PC_ParsedData
Private currentID_ As Long

Public Function Init(theDoc As Word.Document, iDetector As Object, Optional nStart& = 0)
  Set selection_ = theDoc.Range(nStart, nStart)
  Set detector_ = iDetector
  
  start_ = selection_.Paragraphs.First.Range.Start
  Dim sText$: sText = selection_.Paragraphs.First.Range.Text
  Set data_ = detector_.ExtractFragments(sText)
  currentID_ = IIf(data_.Count = 0, 0, NO_CURRENT_ID)
End Function

Public Function Range() As Word.Range
  Set Range = selection_
End Function

Public Function Fragment() As PC_Fragment
  If currentID_ <> NO_CURRENT_ID Then _
    Set Fragment = data_.data_.Item(currentID_)
End Function

Public Function MoveNext() As Word.Range
  If currentID_ = NO_CURRENT_ID Then
    If Not InitNext Then _
      Exit Function
  ElseIf currentID_ = data_.Count Then
    If Not NextChunk Then _
      Exit Function
    currentID_ = 1
  Else
    currentID_ = currentID_ + 1
  End If
  
  Set selection_ = RangeForFragment(data_.data_.Item(currentID_))
  Set MoveNext = selection_
End Function

Public Function MovePrev() As Word.Range
  If currentID_ = NO_CURRENT_ID Then
    If Not InitPrev Then _
      Exit Function
  ElseIf currentID_ <= 1 Then
    If Not PrevChunk Then _
      Exit Function
    currentID_ = data_.Count
  Else
    currentID_ = currentID_ - 1
  End If
  
  Set selection_ = RangeForFragment(data_.data_.Item(currentID_))
  Set MovePrev = selection_
End Function

' ======
Private Function RangeForFragment(iFragment As PC_Fragment) As Word.Range
  Dim iRange As Word.Range: Set iRange = selection_.Document.Range(start_, start_)
  Call iRange.MoveStart(wdCharacter, iFragment.start_)
  Call iRange.Collapse(wdCollapseStart)
  Call iRange.MoveEnd(wdCharacter, iFragment.end_ - iFragment.start_)
  Set RangeForFragment = iRange
End Function

Private Function InitNext() As Boolean
  Dim nTarget&: nTarget = selection_.End
  Dim iRange As Word.Range
  Dim nFragment&
  For nFragment = 1 To data_.Count Step 1
    Set iRange = RangeForFragment(data_.data_.Item(nFragment))
    If iRange.Start >= nTarget Then
      InitNext = True
      currentID_ = nFragment
      Exit Function
    End If
  Next nFragment
  
  If Not NextChunk Then _
    Exit Function
  InitNext = True
  currentID_ = 1
End Function

Private Function InitPrev() As Boolean
  Dim nTarget&: nTarget = selection_.Start
  Dim iRange As Word.Range
  Dim nFragment&
  For nFragment = data_.Count To 1 Step -1
    Set iRange = RangeForFragment(data_.data_.Item(nFragment))
    If iRange.End <= nTarget Then
      InitPrev = True
      currentID_ = nFragment
      Exit Function
    End If
  Next nFragment
  
  If Not PrevChunk Then _
    Exit Function
  InitPrev = True
  currentID_ = data_.Count
End Function

Private Function NextChunk() As Boolean
  Dim iRange As Word.Range: Set iRange = selection_.Document.Range(start_, start_)
  Dim oldStart&
  Do
    oldStart = iRange.Start
    Call iRange.Move(wdParagraph, 1)
    If iRange.Start = oldStart Or iRange.Start + 1 >= selection_.Document.Range.End Then
      NextChunk = False
      Exit Function
    End If
    Dim sText$: sText = iRange.Paragraphs.First.Range.Text
    Dim iData As PC_ParsedData: Set iData = detector_.ExtractFragments(sText)
    If Not iData.IsEmpty Then
      NextChunk = True
      start_ = iRange.Start
      Set data_ = iData
      Exit Function
    End If
  Loop
End Function

Private Function PrevChunk() As Boolean
  Dim iRange As Word.Range: Set iRange = selection_.Document.Range(start_, start_)
  Do
    If iRange.Start = 0 Then
      PrevChunk = False
      Exit Function
    End If
    Call iRange.Move(wdParagraph, -1)
    Dim sText$: sText = iRange.Paragraphs.First.Range.Text
    Dim iData As PC_ParsedData: Set iData = detector_.ExtractFragments(sText)
    If Not iData.IsEmpty Then
      PrevChunk = True
      start_ = iRange.Start
      Set data_ = iData
      Exit Function
    End If
  Loop
End Function
