Attribute VB_Name = "Main"
Option Explicit

Public Function ExportAllData(iParamData() As String) As Variant
  Dim oData As New Collection
  Dim params As New ExtractionOptions: Call params.FromFlatData(iParamData)
  
  Dim iDoc As Word.Document: Set iDoc = Word.ActiveDocument
  Dim oItem As Collection
  
  On Error GoTo RETURN_EMPTY
  Dim iDetector As Object: Set iDetector = PTools().Detector(params.detector_, params.param_)
  Dim oIterator As New IteratorDetected: Call oIterator.Init(iDoc, iDetector)
  Do While Not oIterator.MoveNext Is Nothing
    Set oItem = New Collection
    Call oItem.Add(oIterator.Range.Start)
    Call oItem.Add(oIterator.Range.End)
    Call oItem.Add(IIf(oIterator.Fragment.type_ <> 0, oIterator.Fragment.type_, params.loadCategory_))
    Call oItem.Add(SubstituteWhitespace(oIterator.Range.Text))
    
    Call oData.Add(oItem)
  Loop
  On Error GoTo 0

RETURN_EMPTY:
  ExportAllData = ConverToFlat2D(oData)
End Function

Public Sub RunNextFragment()
  Dim rSelection As Word.Range: Set rSelection = Word.Selection.Range
  Dim iDetector As Object: Set iDetector = PTools().Detector(GetUIState.detector_, GetUIState.detectionParam_)
  Dim oIterator As New IteratorDetected: Call oIterator.Init(rSelection.Document, iDetector, rSelection.End)
  If Not oIterator.MoveNext Is Nothing Then
    Call oIterator.Range.Select
   Else
    Call VBA.Beep
  End If
End Sub

Public Sub RunPrevFragment()
  Dim rSelection As Word.Range: Set rSelection = Word.Selection.Range
  Dim iDetector As Object: Set iDetector = PTools().Detector(GetUIState.detector_, GetUIState.detectionParam_)
  Dim oIterator As New IteratorDetected: Call oIterator.Init(rSelection.Document, iDetector, rSelection.Start)
  If Not oIterator.MovePrev Is Nothing Then
    Call oIterator.Range.Select
  Else
    Call VBA.Beep
  End If
End Sub

Public Sub RunMarkWord()
  Dim iDoc As Word.Document: Set iDoc = Word.ActiveDocument
  Dim nCount&: nCount = MarkAllDetected(iDoc, GetUIState.detector_, GetUIState.color_, GetUIState.detectionParam_)
  Call iDoc.Application.ScreenRefresh
  Call UserInteraction.ShowMessage(IM_MARKDOWN_COMPLETE, nCount)
End Sub

Public Sub RunMarkParagraph()
  Dim iDoc As Word.Document: Set iDoc = Word.ActiveDocument
  Dim nCount&: nCount = MarkAllParagraphs(iDoc, GetUIState.detector_, GetUIState.color_, GetUIState.detectionParam_)
  Call iDoc.Application.ScreenRefresh
  Call UserInteraction.ShowMessage(IM_MARKDOWN_COMPLETE, nCount)
End Sub

Public Sub RunBrowseFile()
  Dim sFile$: sFile = UserInteraction.PromptFileFilter(ActiveDocument.Path & "\", "All files", "*.*")
  If sFile = vbNullString Then _
    Exit Sub
  
  GetUIState.detectionParam_ = sFile
  GetUIState.ribbon_.Value.Invalidate
End Sub

Public Sub RunMorphoParse()
  Dim sText$: sText = EnsureSelectedText()
  If sText = vbNullString Then _
    Exit Sub
  Dim sTags$: sTags = MorphoParse(sText)
  Call UserInteraction.ShowMessage(IM_PARSE_MORPHO, sTags)
End Sub

Public Sub RunContextInfo()
  Dim sText$: sText = EnsureSelectedText()
  If sText = vbNullString Then _
    Exit Sub
    
  Dim iDetector As Object: Set iDetector = PTools().Detector(GetUIState.detector_, GetUIState.detectionParam_)
  If Not iDetector.Test(sText) Then
    Call UserInteraction.ShowMessage(IM_TEST_FAILED, DetectorToStr(GetUIState.detector_))
    Exit Sub
  End If
  
  Dim iParser As Object: Set iParser = PTools().Parser(GetUIState.detector_, GetUIState.detectionParam_)
  If iParser Is Nothing Then
    Call UserInteraction.ShowMessage(IM_NO_PARSER_DATA, DetectorToStr(GetUIState.detector_))
    Exit Sub
  End If
  If Not iParser.Parse(sText) Then
    Call UserInteraction.ShowMessage(IM_NO_PARSER_DATA, DetectorToStr(GetUIState.detector_))
    Exit Sub
  End If
  
  Call UserInteraction.ShowMessage(IM_PARSED_DESCRIPTION, sText, DetectorToStr(GetUIState.detector_), iParser.GetDataDescription())
End Sub

Public Sub RunTransformSingle()
  Dim rSelection As Word.Range: Set rSelection = WordAdjustRange(Word.Selection.Range)
  Call rSelection.MoveEndWhile(" ", -1)
  Dim sText$: sText = VBA.Trim(rSelection.Text)
  If sText = vbNullString Then
    Call UserInteraction.ShowMessage(EM_SELECTION_EMPTY)
    Exit Sub
  End If
  
  Dim iDetector As Object: Set iDetector = PTools().Detector(GetUIState.detector_, GetUIState.detectionParam_)
  If Not iDetector.Test(sText) Then
    Call UserInteraction.ShowMessage(IM_TEST_FAILED, DetectorToStr(GetUIState.detector_))
    Exit Sub
  End If
  
  Dim iParser As Object: Set iParser = PTools().Parser(GetUIState.detector_, GetUIState.detectionParam_)
  If iParser Is Nothing Then _
    Exit Sub
  
  Dim sNewText$: sNewText = iParser.Transform(sText, GetUIState.transformParam_)
  If sText <> sNewText Then _
    rSelection.Text = sNewText
  Call rSelection.Select
End Sub

Public Sub RunTransformAll()
  Dim iDoc As Word.Document: Set iDoc = Word.ActiveDocument
  
  Dim nCount&: nCount = TransformAll(iDoc, GetUIState.GetOptions)
  
  Call UserInteraction.ShowMessage(IM_TRANSFORM_COMPLETE, nCount)
End Sub

Public Sub RunExportRanges()
  Dim iDoc As Word.Document: Set iDoc = Word.ActiveDocument

  Dim xlApp As New API_XLWrapper
  Dim outWB As Excel.Workbook: Set outWB = xlApp.NewDocument
  Call xlApp.PauseUI
  
  Call OutputRanges(iDoc, outWB.Sheets(1), GetUIState.GetOptions)
  
  Call xlApp.ResumeUI
  
  Call UserInteraction.ShowMessage(IM_EXPORT_COMPLETE)
End Sub

Public Sub RunExportAll()
  Dim iDoc As Word.Document: Set iDoc = Word.ActiveDocument

  Dim xlApp As New API_XLWrapper
  Dim outWB As Excel.Workbook: Set outWB = xlApp.NewDocument
  Call xlApp.PauseUI
  
  Call OutputAllData(iDoc, outWB.Sheets(1), GetUIState.GetOptions)
  
  Call xlApp.ResumeUI
  
  Call UserInteraction.ShowMessage(IM_EXPORT_COMPLETE)
End Sub

Public Sub RunHelp()
  MsgBox "TODO"
End Sub

' ======
Private Function EnsureSelectedText() As String
  Dim rSelection As Word.Range: Set rSelection = WordAdjustRange(Word.Selection.Range)
  Dim sText$: sText = VBA.Trim(rSelection.Text)
  If sText = vbNullString Then _
    Call UserInteraction.ShowMessage(EM_SELECTION_EMPTY)
  EnsureSelectedText = sText
End Function
