Attribute VB_Name = "MainImpl"
Option Explicit

Public Function MarkAllDetected(iDoc As Word.Document, nDetector As TDetector, iColor As TColor, sParam$) As Long
  Dim nColor&: nColor = ColorToColorIndex(iColor)
  Dim iDetector As Object: Set iDetector = PTools().Detector(nDetector, sParam)
  Dim oIterator As New IteratorDetected: Call oIterator.Init(iDoc, iDetector)
  Dim iRange As Word.Range
  Dim fragmentColor&
  Dim nCount&: nCount = 0
  Do
    Set iRange = oIterator.MoveNext
    If iRange Is Nothing Then _
      Exit Do
      
    Dim nType&: nType = oIterator.Fragment.type_
    fragmentColor = IIf(nType = 0, 0, ColorToColorIndex(TColor.[_First] + ((nType - 1) Mod TColor.[_Last])))
    If nCount = 0 Then
      Dim oldColor&: oldColor = iRange.HighlightColorIndex
      If oldColor = nColor Or (oldColor = fragmentColor And fragmentColor <> 0) Then _
        nColor = wdAuto
    End If
    If nColor <> wdAuto And fragmentColor <> 0 Then _
      nColor = fragmentColor
    iRange.HighlightColorIndex = nColor
    nCount = nCount + 1
  Loop
  MarkAllDetected = nCount
End Function

Public Function MarkAllParagraphs(iDoc As Word.Document, nDetector As TDetector, iColor As TColor, sParam$) As Long
  Dim nColor&: nColor = ColorToColorIndex(iColor)
  Dim iDetector As Object: Set iDetector = PTools().Detector(nDetector, sParam)
  Dim oIterator As New IteratorDetected: Call oIterator.Init(iDoc, iDetector)
  Dim iRange As Word.Range
  Dim nCount&: nCount = 0
  Dim oldStart&: oldStart = -1
  Do
    Set iRange = oIterator.MoveNext
    If iRange Is Nothing Then _
      Exit Do
    If nCount = 0 And iRange.HighlightColorIndex = nColor Then _
      nColor = wdAuto
    iRange.Paragraphs.First.Range.HighlightColorIndex = nColor
    If oldStart <> iRange.Start Then
      oldStart = iRange.Start
      nCount = nCount + 1
    End If
  Loop
  MarkAllParagraphs = nCount
End Function

Public Function MorphoParse(sText$) As String
  MorphoParse = AccessPython.CallFunction(PY_MODULE_TEXT, "parse", Array(sText, ""))
End Function

Public Function ConverToFlat2D(oData As Collection) As Variant
  Dim iData() As Variant
  If oData.Count = 0 Then
    ConverToFlat2D = iData
    Exit Function
  End If
  
  ReDim iData(0 To oData.Count - 1, 0 To oData.Item(1).Count - 1)
  Dim nRow&: nRow = 0
  Dim oItem As Object
  Dim vElement As Variant
  For Each oItem In oData
    Dim nCol&: nCol = 0
    For Each vElement In oItem
      iData(nRow, nCol) = vElement
      nCol = nCol + 1
    Next vElement
    nRow = nRow + 1
  Next oItem
  
  ConverToFlat2D = iData
End Function

Public Function TransformAll(iDoc As Word.Document, iOptions As ExtractionOptions) As Long
  Dim iParser As Object: Set iParser = PTools().Parser(iOptions.detector_, iOptions.param_)
  If iParser Is Nothing Then _
    Exit Function
  Dim iDetector As Object: Set iDetector = PTools().Detector(iOptions.detector_, iOptions.param_)
  Dim oIterator As New IteratorDetected: Call oIterator.Init(iDoc, iDetector, iDoc.Range.End - 1)
  
  Dim nCount&: nCount = 0
  Do While Not oIterator.MovePrev Is Nothing
    Dim sText$: sText = oIterator.Range.Text
    Dim sNewText$: sNewText = iParser.Transform(sText, iOptions.transform_)
    If sText <> sNewText Then
      oIterator.Range.Text = sNewText
      Call oIterator.Init(iDoc, iDetector, oIterator.Range.Start)
      nCount = nCount + 1
    End If
  Loop
  TransformAll = nCount
End Function

Public Function OutputRanges(iSource As Word.Document, wsOut As Excel.Worksheet, iOptions As ExtractionOptions)
  Dim iDetector As Object: Set iDetector = PTools().Detector(iOptions.detector_, iOptions.param_)
  Dim oIterator As New IteratorDetected: Call oIterator.Init(iSource, iDetector)
  
  With wsOut
    .Cells(1, S_E_ID) = "ID"
    .Cells(1, S_E_START) = ""
    .Cells(1, S_E_FINISH) = ""
    .Cells(1, S_E_TYPE) = ""
  End With
  
  Dim nRow&: nRow = 2
  Do While Not oIterator.MoveNext Is Nothing
    With wsOut
      .Cells(nRow, S_E_ID) = nRow - 1
      .Cells(nRow, S_E_START) = oIterator.Range.Start
      .Cells(nRow, S_E_FINISH) = oIterator.Range.End
      .Cells(nRow, S_E_TYPE) = IIf(oIterator.Fragment.type_ = 0, iOptions.loadCategory_, oIterator.Fragment.type_)
    End With
    nRow = nRow + 1
  Loop
End Function

Public Function OutputAllData(iSource As Word.Document, wsOut As Excel.Worksheet, iOptions As ExtractionOptions)
  Dim iDetector As Object: Set iDetector = PTools().Detector(iOptions.detector_, iOptions.param_)
  Dim iParser As Object: Set iParser = PTools().Parser(iOptions.detector_, iOptions.param_)
  Dim oIterator As New IteratorDetected: Call oIterator.Init(iSource, iDetector)
  
  With wsOut
    .Cells(1, S_E_ID) = "ID"
    .Cells(1, S_E_START) = ""
    .Cells(1, S_E_FINISH) = ""
    .Cells(1, S_E_TYPE) = ""
    .Cells(1, S_E_TEXT) = ""
  End With
  
  Dim nRow&: nRow = 2
  Dim iData As Collection
  Do While Not oIterator.MoveNext Is Nothing
    Dim sText$: sText = oIterator.Range.Text
    With wsOut
      .Cells(nRow, 1) = nRow - 1
      .Cells(nRow, 2) = oIterator.Range.Start
      .Cells(nRow, 3) = oIterator.Range.End
      .Cells(nRow, 4) = IIf(oIterator.Fragment.type_ = 0, iOptions.loadCategory_, oIterator.Fragment.type_)
      .Cells(nRow, 5) = sText
    End With
    If iParser Is Nothing Then _
      GoTo NEXT_ROW
    If Not iParser.Parse(sText) Then _
      GoTo NEXT_ROW
    Set iData = iParser.GetData()
    If iData.Count = 0 Then _
      GoTo NEXT_ROW
      
    Dim nCol&: nCol = S_E_DATA
    Dim vItem As Variant
    For Each vItem In iData
      wsOut.Cells(nRow, nCol) = vItem
      nCol = nCol + 1
    Next vItem
    
    If nRow <> 2 Then _
      GoTo NEXT_ROW
    Dim iHeader As Scripting.Dictionary: Set iHeader = iParser.GetDataDescription()
    nCol = S_E_DATA
    For Each vItem In iHeader.Keys
      wsOut.Cells(1, nCol) = vItem
      nCol = nCol + 1
    Next vItem
    
NEXT_ROW:
    nRow = nRow + 1
  Loop
End Function
