Attribute VB_Name = "MainImpl"
Option Explicit
Option Private Module

Public Function EnsureAddinInstalled() As Boolean
  Dim sPath$: sPath = Environ("APPDATA") & "\Microsoft\Word\STARTUP\" & ADDIN_FILENAME
  Dim fso As New Scripting.FileSystemObject
  EnsureAddinInstalled = fso.FileExists(sPath)
  If Not EnsureAddinInstalled Then _
    Call UserInteraction.ShowMessage(EM_NO_ADDIN)
End Function

Public Function ClearAll()
  SourceSheet.UsedRange.Offset(1).ClearContents
  DataSheet.UsedRange.Offset(1).ClearContents
End Function

Private Function ScanDocsFolder(target$) As Collection
  Set ScanDocsFolder = New Collection
  
  Dim sFolder$: sFolder = target & "\"
  Dim sFile$: sFile = Dir(sFolder & DOC_MASK)
  Do While Len(sFile) > 0 And sFile <> sFolder
    Call ScanDocsFolder.Add(sFolder & sFile)
    sFile = Dir
  Loop
End Function

Public Function ProcessImports(sPath$, bIsFolder As Boolean, params As ExtractionOptions, Optional bSilent As Boolean = False) As Long
  Dim wordApp As New API_WordWrapper: Call wordApp.CreateApplication(bIsVisible:=False)
  If bSilent Then _
    Call wordApp.DisableMessages
    
  Dim xlUI As New API_XLWrapper: Call xlUI.SetApplication(ThisWorkbook.Application)
  Call xlUI.PauseUI
  ThisWorkbook.Application.DisplayAlerts = False
  
  Dim nCount&: nCount = 0
  Dim theDoc As Word.Document
  If Not bIsFolder Then
    Set theDoc = wordApp.OpenDocument(sPath, bReadOnly:=True)
    If theDoc Is Nothing Then _
      GoTo SAFE_EXIT
    nCount = nCount + ScanDataFrom(wordApp, params)
    Call wordApp.ReleaseDocument
  Else
    Dim docs As Collection: Set docs = ScanDocsFolder(sPath)
    Call CSE_ProgressBar.Init(" ", maxVal:=docs.Count - 1, canInterrupt:=True)
    If Not bSilent Then _
      Call CSE_ProgressBar.Show
    
    Dim sFile As Variant
    For Each sFile In docs
      Set theDoc = wordApp.OpenDocument(CStr(sFile), bReadOnly:=True)
      If theDoc Is Nothing Then _
        GoTo SAFE_EXIT
        
      theDoc.Range.NoProofing = True
        
      CSE_ProgressBar.Description = theDoc.Name
      nCount = nCount + ScanDataFrom(wordApp, params)
      Call wordApp.ReleaseDocument(bCloseApplication:=False)
      
      Call CSE_ProgressBar.IncrementA
      If CSE_ProgressBar.Interrupted Then _
        Exit For
    Next sFile
    
    Call Unload(CSE_ProgressBar)
  End If

SAFE_EXIT:
  ThisWorkbook.Application.DisplayAlerts = True
  Call xlUI.ResumeUI
  Call wordApp.ReleaseApplication
  ProcessImports = nCount
End Function

Public Function FollowLinkAt(target&, nStart&, nEnd&)
  Dim sPath$: sPath = SourceFileFor(target)
  If sPath = vbNullString Then
    Call UserInteraction.ShowMessage(EM_INVALID_SOURCE_ID)
    Exit Function
  End If
  
  Dim fso As New Scripting.FileSystemObject
  If Not fso.FileExists(sPath) Then
    Call UserInteraction.ShowMessage(EM_INVALID_PATH, sPath)
    Exit Function
  End If
  
  Dim wordApp As New API_WordWrapper
  Dim theDoc As Word.Document: Set theDoc = wordApp.OpenDocument(sPath)
  If theDoc Is Nothing Then _
    Exit Function

  If theDoc.Range.End < nEnd Then
    Call UserInteraction.ShowMessage(EM_INVALID_LINK, sPath)
    Exit Function
  End If

  Dim targetRng As Word.Range: Set targetRng = theDoc.Range(nStart, nEnd)
  Call targetRng.Select
  Call theDoc.ActiveWindow.ScrollIntoView(targetRng)

  Call wordApp.Application.Activate
End Function

' ======
Private Function ScanDataFrom(target As API_WordWrapper, params As ExtractionOptions) As Long
  Dim dataSht As Excel.Worksheet: Set dataSht = DataSheet
  Dim srcSht As Excel.Worksheet: Set srcSht = SourceSheet
  Dim theTimer As New API_Timer
  
  Call theTimer.Start
  Call target.Document.Activate
  
  Dim srcID&: srcID = AddSource(target.Document)
  Dim srcRow: srcRow = RowFor(srcID)
  Dim nCount&: nCount = 0
  Dim newData As Variant
  
  Call target.Document.Activate
  On Error GoTo EXIT_FUNCTION
  newData = target.Run(FUNC_PARSE, params.AsFlatData)
  On Error GoTo 0
  If ArraySize(newData) <= 0 Then _
    Exit Function
  
  Dim nRow&: nRow = dataSht.Columns(DS_ID).Find(vbNullString, LookAt:=xlWhole).Row
  Dim nDataRow&
  Dim nDataCol&
  For nDataRow = LBound(newData, 1) To UBound(newData, 1) Step 1
    dataSht.Cells(nRow, DS_ID) = srcID
    Dim nCol&: nCol = DS_ID + 1
    For nDataCol = LBound(newData, 2) To UBound(newData, 2) Step 1
      dataSht.Cells(nRow, nCol) = newData(nDataRow, nDataCol)
      nCol = nCol + 1
    Next nDataCol
    
    nRow = nRow + 1
    nCount = nCount + 1
  Next nDataRow
  
  srcSht.Cells(srcRow, SS_TIME) = theTimer.TimeElapsed
  srcSht.Cells(srcRow, SS_COUNT) = nCount
  srcSht.Cells(srcRow, SS_NAME) = target.Document.Name
  srcSht.Cells(srcRow, SS_PARS) = target.Document.Paragraphs.Count
  
EXIT_FUNCTION:
  ScanDataFrom = nCount
End Function
